#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const API_DIR = "/var/www/html/portal/api/";

const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP-base.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile('from_device');

// Process arguments
if($argc > 0){
    foreach($argv as $arg){
        $args = explode('=',$arg);
        switch($args[0]){
            case '--help':
                return displayUsage();
            case '--log':
                $Log->setLogFile($args[1]);
                break;
        }
    }
}

$pid = pcntl_fork();
if ($pid == -1) {
    $Log->logError('Could not daemonize process');
    return 1;
} else if ($pid) {
    return 0;
} else {

    $hosts = $Config->getHosts();
    foreach ($hosts as $entry) {
        $AMQP = new AMQP($entry['host'], $Config->getUser($entry['host']));
        $AMQP->setPort($Config->getPort());
        $AMQP->connect($Config->useSSL());
        $channel = $AMQP->createChannel();
        $channel->setPrefetchCount(NULL);        // testing - get all pre-fetched
        $exchange = $AMQP->createExchange();

        try {
            $routing_key = Configuration::DEVICE_MONITORING_ROUTE;
            $queue = $AMQP->createQueue($routing_key);
            $Log->logInfo("Waiting on messages....\n");
            $queue->consume(function ($envelope, $queue) {
                global $Log;
                processMessage($envelope);
                $queue->ack($envelope->getDeliveryTag());
                $Log->logInfo("Waiting on messages....\n");
            });

            $AMQP->disconnect();
        } catch (Exception $ex) {
            $Log->logError($ex);
        }
    }
}


$i = 0;
$conn = null;
function processMessage($envelope) {
    global $i;
    global $Log;
    global $conn;
    $Log->logInfo("Received Message $i: " . $envelope->getBody());
    
    $jsonString = $envelope->getBody();
    $jsonObj = json_decode($jsonString, true);
    if (isset($jsonObj['asset_tag'])) {
        $asset_tag = $jsonObj['asset_tag'];
        $command = $jsonObj["meta_command"];
        $timestamp = $jsonObj["meta_timestamp"];
        $msg = sprintf("Message %d, Device ID is %s, Command is %s\n", $i, $asset_tag, $command);
        $Log->logInfo($msg);
        $commands = new Commands($command, $conn);
        $commands->run($timestamp, $asset_tag, $jsonObj);
        $conn = $commands->getConnection();
        //processCommand($command, $timestamp, $asset_tag, $jsonObj);
    }
    $i++;
    if ($i > 10) {
        echo "greater than 10 messages";
     }
    return true;
}


$conn = NULL;
function save_monthly($timestamp, $asset_tag, $data) {
    global $conn;
    global $Log;
    
    if ($conn == NULL) {
        $conn = pg_connect("port=5432 dbname=mgportal user=postgres connect_timeout=30");
    }

    if ($conn !== false) {
        $result = pg_query($conn, "SELECT * from mgportal.devices");
        if ($result !== false) {
            $found = false;
            while ($row = pg_fetch_row($result)) {
                if ($asset_tag == $row[1]) {
                    $myID = $row[0];
                    $found = true;
                    break;
                }
            }
            if (!$found) {
                $licenseClass = $data["license_class"];
                $marketingName = $data["license_mkt_name"];
                $featureString = $data["license_feature"];
                $installDate = $data["install_date"];
                $query = "INSERT INTO mgportal.devices (unique_identifier, license_class, license_mkt, license_features, install_date) VALUES ('%s', '%s', '%s', '%s', '%s')";
                $pg = sprintf($query, $asset_tag, $licenseClass, $marketingName, $featureString, $installDate);
                $result = pg_query($conn, $pg);
                $myID = 1;
            }
        }

        if (isset($myID)) {
            $query = "INSERT INTO mgportal.monthly (date, protected_assets, unique_identifier) ";
            $query .= "VALUES (%d, %d, '%s')";
            $pg = sprintf($query, $timestamp, $data['protected_assets'], $asset_tag);
            pg_query($conn, $pg);
            //printf("Query is %s\n", $pg);
        }
        pg_close($conn);
    } else {
        $Log->logError("Error creating database connection, information not logged");
    }
}

?>
