#!/bin/bash

#
# Generate server and client certificates for RabbitMQ
#

CERT_BASE=/usr/local/mgportal/certs
TEST_CA=${CERT_BASE}/testca

generateCertificate() {

	local certType=$1

	cd $CERT_BASE
	mkdir -p ${certType}
	cd ${certType}


	# generate key and request
	openssl genrsa -out key.pem 2048
	openssl req -new -key key.pem -out req.pem -outform PEM -subj /CN=$(hostname)/O=${certType}/ -nodes

	# sign with the CA
	cd $TEST_CA
	openssl ca -config openssl.cnf -in ../${certType}/req.pem -out ../${certType}/cert.pem -notext -batch -extensions ${certType}_ca_extensions

}

createCA() {
	cd $TEST_CA
	mkdir -p certs private
	chmod 700 private
	echo 01 > serial
	touch index.txt

	# generate key and certs that CA will use
	openssl req -x509 -config openssl.cnf -newkey rsa:2048 -days 730 -out cacert.pem -outform PEM -subj /CN=MyTestCA/ -nodes
}

if [ -d $TEST_CA ]
then
	if [ ! -f $TEST_CA/cacert.pem ] || [ "$1" = "force" ]
	then
		createCA
	else
		echo "CA has already been created.  Use \"force\" option to re-create."
	fi

	if [ ! -f $CERT_BASE/server/key.pem ] || [ "$1" = "force" ]
	then
		generateCertificate server
	else
		echo "server certificates exist.  Use \"force\" option to re-create."
	fi

	if [ ! -f $CERT_BASE/client/key.pem ] || [ "$1" = "force" ]
	then
		generateCertificate client
	else
		echo "client certificates exist.  Use \"force\" option to re-create."
	fi
else
	echo "Error: Certificate directory $TEST_CA does not exist"
	exit 255
fi
