#!/bin/bash

MGPORTAL_DIR=/usr/local/mgportal
. $MGPORTAL_DIR/bin/defines.incl

mgportal_usage() {

echo "
usage: $0 [create|update] <logfile>
	where logfile is optional.
"
exit 1
}

if [ "$#" -eq 0 ]
then
        mgportal_usage
fi

option=`echo $1 | tr '[:lower:]' '[:upper:]'`
if [ $option = CREATE ]
then
	if [ ! -d "$VENDOR_DB_DIR" ]; then
		mkdir $MGPORTAL_DIR/data

		echo "Creating database..." >>$LOG
		chown postgres:postgres $MGPORTAL_DIR/data
		sudo -u postgres initdb -D $MGPORTAL_DIR/data -E UTF-8 --lc-collate='C' --lc-ctype='C'>>$LOG 2>&1

		echo "Starting postmaster..." >>$LOG
		sudo -iu postgres pg_ctl start -D $MGPORTAL_DIR/data -l $MGPORTAL_DIR/data/logfile -w  >>$LOG 2>&1
		ret=$?
		if [ $ret -ne 0 ]
		then
			echo "ERROR: pg_ctl start returned $ret." >>$LOG
			error_exit $ret $option
		fi
	else
		echo "Vendor database directory already exists..." >>$LOG
	fi


	sudo -iu postgres createdb mgportal  >>$LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: createdb returned $ret." >>$LOG
        	error_exit $ret $option
	fi
fi

if [ $option = UPDATE ]
then
	$MGPORTAL_DIR/bin/start_database.sh >> $LOG 2>&1
	ret=$?
	if [ $ret -ne 0 ]
	then
		echo "ERROR: start returned $ret." >>$LOG
		error_exit $ret $option
	fi
fi


for sql_file in base.sql update.sql
do
        echo "Loading $sql_file..." >>$LOG
        sudo -iu postgres psql -v ON_ERROR_STOP=TRUE -f $MGPORTAL_SCHEMA/$sql_file mgportal >>$LOG 2>&1
        ret=$?
        if [ $ret -ne 0 ]
        then
                echo "ERROR: Load of $sql_file returned $ret." >>$LOG
                error_exit $ret $option
        else
                echo "Load of $sql_file was successful." >>$LOG
        fi
done

