<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class Alerts extends BaseAPI {

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which, $openOnly, $isReport = false){
        $query = "SELECT device_id, created, updated, id, message, resolved, severity, sname, source_id, source_name, system_id  FROM mgportal.alerts";
        if ($which !== null) {
            $query .= " WHERE device_id=" . $which;
        }
        if ($openOnly) {
            $qualifier = $which !== null ? " AND" : " WHERE";
            $query .= $qualifier . " resolved=0";
        }

        $alerts = array();

        $dbAlerts = Database::db_query($this->conn, $this->Log, $query);
        if ($dbAlerts !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbAlerts)){
                $alert = array(    'device_id' => $row[0],
                    'created' => $row[1],
                    'updated' => $row[2],
                    'id' => $row[3],
                    'message' => $row[4],
                    'resolved' => $row[5],
                    'severity' => $row[6],
                    'system_name' => $row[7],
                    'source_id' => $row[8],
                    'source_name' => $row[9],
                    'system_id' => $row[10]);
                $alerts[] = $alert;
            }
        }

        $alertsReturn = array();
        if ($isReport) {
            $alertsReturn['data'] = $alerts;
        } else {
            $alertsReturn['alerts'] = $alerts;
        }
        return $alertsReturn;
    }

    public function addAlerts($data, $clearFirst){
        $result = false;
        $device_id = -1;
        if (!$this->validateAsset($data, $result, $device_id)) {
            return $result;
        }

        if (!is_array($data['data'])) {
            $message = "Data not provided / empty";
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            // check to see if clear first.
            if ($clearFirst) {
                $result = $this->clearAlerts($device_id, true);
                if ($result !== true) {
                    $result = array(
                        'result' => false,
                        'message' => 'Error clearing previous alerts.'
                    );
                    return $result;
                }
            }
            // insert into database
            $fulldata = $data['data'];
            $timestamp = $data['meta_timestamp'];
            foreach ($fulldata as $data) {
                // check if array
                if (!is_array($data)) {
                    // do nothing with this one
                    $message = "Data not provided / empty";
                    $this->Log->logError($message);
                    $result = array(
                        'result' => false,
                        'message' => $message
                    );
                    // if array - continue
                } else {
                    // build query
                    $id = isset($data['id']) ? $data['id'] : null;
                    $message = isset($data['message']) ? $data['message'] : null;
                    $created = isset($data['created']) ? $data['created'] : null;
                    $resolved = isset($data['resolved']) ? $data['resolved'] : 0;
                    $severity = isset($data['severity']) ? $data['severity'] : null;
                    $sname = isset($data['sname']) ? $data['sname'] : null;
                    $source_id = isset($data['source_id']) ? $data['source_id'] : null;
                    $source_name = isset($data['source_name']) ? $data['source_name'] : null;
                    $system_id = isset($data['system_id']) ? $data['system_id'] : null;
                    $updated = isset($data['updated']) ? $data['updated'] : null;


                    $ins_query = "INSERT INTO mgportal.alerts (device_id, created, id, message, resolved";
                    $ins_query .= ", severity, sname, source_id, source_name, system_id, updated)";

                    $add_val_query = "VALUES (%d, '%s', %d, %s, %d, '%s', %s, %d, '%s', %d, '%s');";
                    $val_query = sprintf($add_val_query, $device_id, $created, $id, Database::db_escape_string($message), $resolved,
                        $severity, Database::db_escape_string($sname), $source_id, $source_name, $system_id, $updated);

                    $query = $ins_query . $val_query;

                    $queryResult = Database::db_query($this->conn, $this->Log, $query);
                    if ($queryResult === false) {
                        $this->Log->logError("Query returned false -- addAlerts");
                        $result = false;
                    }
                }
            }
            $this->setTimestamp('alerts', $timestamp, 0, $device_id);
        }
        return $result;
    }

    public function clearAlerts($deviceId, $okayIfNoMatch = false, $alertId = NULL){
        $query = "DELETE FROM mgportal.alerts";
        if ($deviceId !== null) {
            $query .= " WHERE device_id=" . $deviceId . ';';
        } else if ($alertId !== null) {
            $query .= " WHERE id=" . $alertId . ';';
        } else {
            $result = array(
                'result' => false,
                'message' => "No device or alert id present for clearing prior alerts"
            );
            return $result;
        }

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $result);
            if ($row == null && !$okayIfNoMatch){
                $result = array(
                    'result' => false,
                    'message' => "Error, alert does not exist."
                );
            } else {
                $result = true;
            }
        } else {
            $result = array(
                'result' => false,
                'message' => "Error deleting alerts."
            );
        }
        return $result;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $alerts = new Alerts($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $openOnly = isset($_GET['open']) && $_GET['open'] == 'true';
                $isReport = isset($_GET['type']) && $_GET['type'] == 'report';
                $result = $alerts->get($which, $openOnly, $isReport);
                $http->buildResult($result, "get");
                break;
            case 'post':
                $clearPriorItems = (isset($_GET['refresh']) && $_GET['refresh'] == 1);
                $result = $alerts->addAlerts($http->getData(), $clearPriorItems);
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $alertId = isset($_GET['alert_id']) ? $_GET['alert_id'] : null;
                $result = $alerts->clearAlerts($which, true, $alertId);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}


?>