<?php
/**
 *  Devices API
 */

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class Devices extends BaseAPI {
    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }
    
    public function get($which, $filter) {
        
        $query = "SELECT * FROM mgportal.devices";
        if ($which !== null) {
            $query .= " WHERE unique_identifier='" . $which . "';";
        }
        $assetsArray = array();
        $assets = Database::db_query($this->conn, $this->Log, $query);
        if ($assets !== false) {
            //printf("'Latest Data Date','Asset Tag','Feature String','Install Date', 'VM Count', 'Server Count', 'App Count', 'All Count', 'GB Used', 'VM Replicating', 'All Replicating'\n");
            while ($row = Database::db_fetch_row($this->Log,  $assets)) {
                //printf("asset = %s\n", $row[1]);
                $asset_id = $row[0];
                $asset = $row[1];
                $license_class = $row[2];
                $feature_string = $row[4];
                $install_date = $row[5];

                $query = "SELECT date, vm_count, server_count, app_count, gb_used, vm_replicated, all_replicated, replication_used";
                $query .= " FROM mgportal.protected where device_id='";
                $query .= $asset_id;
                $query .= "' ORDER BY date DESC LIMIT 1";
                $protected = Database::db_query($this->conn, $this->Log, $query);
                if ($protected !== false) {
                    while ($item = Database::db_fetch_row($this->Log,  $protected)) {
                        $timestamp = $item[0];
                        $dateString = date('m/d/Y', $timestamp);
                        
                        $vm_protected = $item[1];
                        $server_protected = $item[2];
                        $app_protected = $item[3];
                        $gb_used = $item[4];
                        $vm_replicated = $item[5];
                        $all_replicated = $item[6];
                        $replication_used = $item[7];
                        
                        $assetsArray[] = array("date" => $dateString,
                                                "asset" => $asset,
                                                "license" => $feature_string,
                                                "vms" => $vm_protected,
                                                "servers" => $server_protected,
                                                "apps" => $app_protected,
                                                "gb_protected" => $gb_used,
                                                "replication_protected" => $replication_used,
                                                "vms_replicated" => $vm_replicated,
                                                "all_replicated" => $all_replicated);
                    }
                }
            }
        }
        return array("protected" => $assetsArray);
    }

    public function getAll() {
        $result = array();
        $query = "SELECT d.id, d.hostname, d.unique_identifier, l.id, l.name, m.id, m.name FROM mgportal.devices d INNER JOIN mgportal.locations l ON d.location_id=l.id ";
        $query .= "INNER JOIN mgportal.managers m ON l.manager_id=m.id;";
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $item = array('device_id' => $row[0],
                    'hostname' => $row[1],
                    'uuid' => $row[2],
                    'location_id' => $row[3],
                    'location_name' => $row[4],
                    'manager_id' => $row[5],
                    'manager_name' => $row[6]);
                $result[] = $item;
            }
        }
        return $result;
    }


    public function put($which, $options, $isManaging = null) {
        $result = false;

        if ($which === null) {
            $result = array(
                'result' => false,
                'message' => 'Device id is not optional for update.'
            );
        } else {
            if (is_array($options)) {
                $query = "UPDATE mgportal.devices ";

                $query .= "SET ";
                if (isset($options['location_id'])) {
                    $query .= "location_id=";
                    $query .= $options['location_id'];
                    $query .= ",";
                }
                if (isset($options['hostname'])) {
                    $query .= "hostname='";
                    $query .= $options['hostname'];
                    $query .= "',";
                }
                if (isset($options['version'])) {
                    $query .= "version='";
                    $query .= $options['version'];
                    $query .= "',";
                }
                if (isset($options['license_class'])) {
                    $query .= "license_class='";
                    $query .= $options['license_class'];
                    $query .= "',";
                }
                if (isset($options['license_mkt'])) {
                    $query .= "license_mkt='";
                    $query .= $options['license_mkt'];
                    $query .= "',";
                }
                if (isset($options['license_feature'])) {
                    $query .= "license_features='";
                    $query .= $options['license_feature'];
                    $query .= "',";
                }
                if (isset($options['install_date'])) {
                    $query .= "install_date='";
                    $query .= $options['install_date'];
                    $query .= "',";
                }
                if (isset($options['credential_id'])) {
                    // If credential ID is 0, set device credential to NULL.
                    $credValue = (int)$options['credential_id'] === 0 ? "NULL" : $options['credential_id'];
                    $query .= "credential_id=";
                    $query .= $credValue;
                }
                if ($isManaging !== null) {
                    $last = substr($query, -1);
                    if ($last !== ",") {
                        $query .= ",";
                    }
                    // Only set activity if it's true or false, not null.  Activity is detected on the client, so comes through a client message/API.
                    $query .= "activity=";
                    $query .= $isManaging ? 1 : 0;
                }
                $query .= " WHERE id=" . $which . ";";

                // put is allowing any id to return success
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logError("Error saving device information");
                    $result = false;
                }
            }
        }
        return $result;
    }

    private function is_device_present($asset_tag) {
        $device_id = false;
        if ($this->conn !== false) {
            $query = "SELECT id, unique_identifier from mgportal.devices WHERE unique_identifier='" . $asset_tag . "';";
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                while ($row = Database::db_fetch_row($this->Log,  $result)) {
                    if ($asset_tag == $row[1]) {
                        $device_id = $row[0];
                        break;
                    }
                }
            }
        }
        return $device_id;
    }

    private function addDevice($asset_tag, $data, $isManaging = false) {

        $result = true;
        $device_id = $this->is_device_present($asset_tag);
        if (!$device_id) {
            $licenseClass = isset($data["license_class"]) ? $data['license_class'] : '';
            $marketingName = isset($data["license_mkt_name"]) ? $data['license_mkt_name'] : '';
            $featureString = isset($data["license_feature"]) ? $data["license_feature"] : '';
            $installDate = isset($data["install_date"]) ? $data["install_date"] : '';
            $hostname = isset($data["hostname"]) ? $data['hostname'] : '';
            $hasSupport = isset($data["has_support_contract"]) ? ($data["has_support_contract"] === true ? "TRUE" : "FALSE") : "FALSE";
            $timestamp = isset($data["meta_timestamp"]) ? $data["meta_timestamp"] : time();
            $locationID = isset($data['location_id']) ? $data['location_id'] : 1;
            $activity = $isManaging ? 1 : 0;
            $query = "INSERT INTO mgportal.devices (unique_identifier, has_support_contract, ";
            $query .= "license_class, license_mkt, license_features, install_date, ";
            $query .= "date_created, hostname, location_id, activity)";
            $query .= " VALUES ('%s', %s, '%s', '%s', '%s', '%s', '%d', '%s', '%d', '%d')";
            $query .= " RETURNING id;";
            $pg = sprintf($query, $asset_tag, $hasSupport, $licenseClass, $marketingName, $featureString, $installDate, $timestamp, $hostname, $locationID, $activity);
            $queryResult = Database::db_query($this->conn, $this->Log, $pg);
            if ($queryResult !== false) {
                $row = Database::db_fetch_row($this->Log,  $queryResult);
                $device_id = $row[0];
                $result = array('result' => true, 'id' => $device_id);
            } else {
                $this->Log->logError("Error saving device information");
            }
        } else {
            // If meta_command is set for device check, update on POST if it exists.
            if (isset($data['meta_command']) && $data['meta_command'] == Configuration::CMD_DEVICE_CHECK) {
                $result = $this->put($device_id, $data, $isManaging);
            } else {
                $this->Log->logError("Device already exists");
                $result = array('result' => false, 'message' => 'Device already exists.');
            }
        }

        return $result;
    }
    
    public function post($data, $hasSupport, $isManaging) {
        $result = true;
        if (isset($data['asset_tag'])) {
            $asset_tag = $data['asset_tag'];
            if (isset($data['meta_command']) && ($data['meta_command'] != Configuration::CMD_DEVICE_CHECK)) {
                $command = $data["meta_command"];
                $timestamp = $data["meta_timestamp"];
                $data["has_support_contract"] = $hasSupport;
                $msg = sprintf("Message: Device ID is %s, Command is %s", $asset_tag, $command);
                $this->Log->logInfo($msg);
                $commands = new Commands($command, $this->conn);
                $commands->run($timestamp, $asset_tag, $data);
            } else {
                $result = $this->addDevice($asset_tag, $data, $isManaging);
            }
        } else {
            $result = array('result' => false, 'message' => 'Asset Tag must be specified when adding device.');
        }
        return $result;
    }

    public function delete($id) {
        if ($id === null) {
            $result = array(
                'result' => false,
                'message' => 'Device id is not optional for delete.'
            );
        } else {
            $query = "DELETE FROM mgportal.devices WHERE id=";
            $query .= $id;
            $query .= " RETURNING id;";
            $queryResult = Database::db_query($this->conn, $this->Log, $query);
            if ($queryResult !== false) {
                $row = Database::db_fetch_row($this->Log,  $queryResult);
                if ($row === null || $row === false) {
                    $result = array(
                        'result' => false,
                        'message' => "Error: device id " . $id . " does not exist"
                    );
                } else {
                    $result = true;
                }
            } else {
                $message = "Error deleting device " . $id;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        }
        return $result;
    }
    
};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $devices = new Devices($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $type = isset($_GET['type']) ? $_GET['type'] : null;
                if ($type === 'all') {
                    $result = $devices->getAll();
                } else {
                    $result = $devices->get($which, null);
                }
                if ($result !== false) {
                    $http->buildSuccess($result);
                } else {
                    $http->buildFailure(false, "Error getting devices");;
                }
                break;
            case 'put':
                $result = $devices->put($which, $http->getData());
                $http->buildResult($result, "put");
                break;
            case 'post':
                // devtype if set is even if managing, odd if monitor only.
                // $managing is true if managing and false if not.
                $managing = isset($_GET['devtype']) ? ((int)$_GET['devtype'] % 2 == 0) : false;
                $result = $devices->post($http->getData(), $http->getDeviceSupportStatus(), $managing);
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $devices->delete($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $message = $http->getError();
    $message = $message == "" ? HTTP::UNAUTHENTICATED_MESSAGE : $message;
    $Log->logError($message);
    $http->buildFailure(false, $message);
}
?>