<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Locations extends BaseAPI {
    const DEFAULT_LOCATION = 1;

    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($location_id = null){

        $query = "SELECT id, manager_id, name, country, street, city, state, zip, phone, latitude, longitude FROM mgportal.locations";
        if ($location_id !== null) {
            $query .= " WHERE id=" . $location_id;
        }

        $locations = array();
        $dbLoc = Database::db_query($this->conn, $this->Log, $query);
        if ($dbLoc !== false){
            while ($row = Database::db_fetch_row($this->Log, $dbLoc)){
                $id = $row[0];
                $manager_id = $row[1];
                $name = $row[2];
                $country = $row[3];
                $street = $row[4];
                $city = $row[5];
                $state = $row[6];
                $zip = $row[7];
                $phone = $row[8];
                $latitude = $row[9];
                $longitude = $row[10];
                $location = array('id' => $id,
                    'manager_id' => $manager_id,
                    'name' => $name,
                    'country' => $country,
                    'street' => $street,
                    'city' => $city,
                    'state' => $state,
                    'zip' => $zip,
                    'phone' => $phone,
                    'latitude' => $latitude,
                    'longitude' => $longitude);
                $locations[] = $location;
            }
        }
        return array("locations" => $locations);
    }

    public function addLocation($data){
        global $Log;

        // must be set
        if (isset($data['manager_id'])){
            $manager_id = $data['manager_id'];
        } else {
            $message = "Manager_id not provided for new location";
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        }
        if (isset($data['name'])){
            $locationName = $data['name'];
        } else {
            $message = "Location name must be provided for new location";
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        }
        $country = isset($data['country']) ? $data['country'] : null;
        $street = isset($data['street']) ? $data['street'] : null;
        $city = isset($data['city']) ? $data['city'] : null;
        $state = isset($data['state']) ? $data['state'] : null;
        $zip = isset($data['zip']) ? $data['zip'] : null;
        $phone = isset($data['phone']) ? $data['phone'] : null;
        $latitude = isset($data['latitude']) ? $data['latitude'] : null;
        $longitude = isset($data['longitude']) ? $data['longitude'] : null;

        $str = "INSERT INTO mgportal.locations (name, country, street, city, state, zip, phone, latitude, longitude, manager_id)";
        $str .= " VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %d) RETURNING id;";
        $query = sprintf($str, Database::db_escape_string($locationName), Database::db_escape_string($country),
                Database::db_escape_string($street), Database::db_escape_string($city),
                Database::db_escape_string($state), Database::db_escape_string($zip),
                Database::db_escape_string($phone),
                Database::db_escape_string($latitude), Database::db_escape_string($longitude), $manager_id);

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log, $result);
            $location_id = $row[0];
        } else {
            $Log->logError("Error saving location information");
            $location_id = false;
        }
        return $location_id;
    }

    public function updateLocation($location_id, $options){
        $result = false;

        if (isset($options['manager_id'])){
            $result = array(
                'result' => false,
                'message' => 'Cannot change manager_id of a location'
            );
            return $result;
        }

        if ($location_id === null) {
            $result = array(
                'result' => false,
                'message' => 'location_id is not optional for update.'
            );
        } else {
            if (is_array($options)) {
                $query = "UPDATE mgportal.locations SET ";
                if (isset($options['name'])) {
                    $query .= "name=" . Database::db_escape_string($options['name']);
                    $query .= ",";
                }
                if (isset($options['country'])) {
                    $query .= "country=" . Database::db_escape_string($options['country']);
                    $query .= ",";
                }
                if (isset($options['street'])) {
                    $query .= "street=" . Database::db_escape_string($options['street']);
                    $query .= ",";
                }
                if (isset($options['city'])) {
                    $query .= "city=" . Database::db_escape_string($options['city']);
                    $query .= ",";
                }
                if (isset($options['state'])) {
                    $query .= "state=" . Database::db_escape_string($options['state']);
                    $query .= ",";
                }
                if (isset($options['zip'])) {
                    $query .= "zip=" . Database::db_escape_string($options['zip']);
                    $query .= ",";
                }
                if (isset($options['phone'])) {
                    $query .= "phone=" . Database::db_escape_string($options['phone']);
                    $query .= ",";
                }
                if (isset($options['latitude'])) {
                    $query .= "latitude=" . Database::db_escape_string($options['latitude']);
                    $query .= ",";
                }
                if (isset($options['longitude'])) {
                    $query .= "longitude=" . Database::db_escape_string($options['longitude']);
                    $query .= ",";
                }
                // remove last comma
                $query = substr($query,0,-1);

                $query .= " WHERE id=" . $location_id . ";";
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logError("Error saving location information");
                }
            }
        }
        return $result;
    }

    public function delete($location_id) {
        $result = true;
        if ($location_id === null) {
            $result = array(
                'result' => false,
                'message' => 'Location id is not optional for delete.'
            );
        } else if ($location_id == Locations::DEFAULT_LOCATION){
            $result = array(
                'result' => false,
                'message' => 'This is the default location for appliances when discovered so cannot be deleted.'
            );
        } else if ($this->hasReferences($location_id)){
            $result = array(
                'result' => false,
                'message' => 'This has appliances assigned to it.  These must be assigned to other locations before this location can be deleted.'
            );
        } else {
            $query = "DELETE FROM mgportal.locations WHERE id=";
            $query .= $location_id;
            $query .= " RETURNING id;";
            $queryResult = Database::db_query($this->conn, $this->Log, $query);
            if ($queryResult !== false) {
                $row = Database::db_fetch_row($this->Log, $queryResult);
                if ($row === null || $row === false) {
                    $result = array(
                        'result' => false,
                        'message' => "Error: location id " . $location_id . " does not exist"
                    );
                } else {
                    $result = $row[0];
                }
            } else {
                $message = "Error deleting location id " . $location_id;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        }
        return $result;
    }

    private function hasReferences($id){
        $result = true;
        // check for devices
        $deviceChildren = $this->getDeviceChildren($id);
        if (count($deviceChildren) == 0) {
            $result = false;
        }
        return $result;
    }

    public function getDeviceChildren($id) {
        $result = array();
        // check for device children
        $query = "SELECT id, unique_identifier, hostname FROM mgportal.devices WHERE location_id=" . $id;
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log, $qResult)){
                $device = array('id' => $row[0], 'uuid' => $row[1], 'hostname' => $row[2]);
                $result[] = $device;
            }
        }
        return $result;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $locations = new Locations($conn, $Log);
        $which = isset($_GET['location_id']) ? $_GET['location_id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $locations->get($which);
                $http->buildResult($result, "get");
                break;
            case 'put':
                $result = $locations->updateLocation($which, $http->getData());
                $http->buildResult($result, "put");
                break;
            case 'post':
                $result = $locations->addLocation($http->getData());
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $locations->delete($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}


?>