<?php
/**
 *  Preferences API
 */


const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");
require_once("./system/" . "password.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Preferences extends BaseAPI {
    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($which, $key) {
        if ($which === null) {
            $result = array(
                'result' => false,
                'message' => 'Username is not optional when getting preferences.'
            );
        } else {
            $query = "SELECT key, value FROM mgportal.preferences";
            $query .= " WHERE username='" . $which . "'";
            if ($key !== null) {
                $query .= ($which === null) ? " WHERE" : " AND";
                $query .= " key='" . $key . "'";
            }
            $query .= ";";
            $preferences = array();

            $dbPreferences = Database::db_query($this->conn, $this->Log, $query);
            if ($dbPreferences !== false) {
                while ($row = Database::db_fetch_row($this->Log,  $dbPreferences)) {
                    $key = $row[0];
                    $value = $row[1];
                    $preference = array($key => $value);
                    $preferences = array_merge($preferences, $preference);
                }
            }
            $result = array("preferences" => $preferences);
        }
        return $result;
    }

    public function update($userName, $options)
    {
        $result = false;

        if ($userName === null) {
            $result = array(
                'result' => false,
                'message' => 'Username is not optional for update.'
            );
        } else {
            if (is_array($options)) {
                foreach ($options as $key => $value) {
                    $query = "UPDATE mgportal.preferences SET ";
                    $query .= "key=" . Database::db_escape_string($key);
                    $query .= ", value=" . Database::db_escape_string($value);
                    $query .= " WHERE username='" . $userName . "';";
                    $this->Log->logDebug("Query is " . $query);
                    $result = Database::db_query($this->conn, $this->Log, $query);
                    if ($result !== false) {
                        $rows = Database::db_affected_rows($this->Log, $result);
                        if ($rows === 0) {
                            // preference not present, add it.
                            $this->Log->logDebug("update result no rows, add preference " . $key);
                            $str = "INSERT INTO mgportal.preferences (username, key, value)";
                            $str .= " VALUES ('%s', %s, %s)";
                            $query = sprintf($str, $userName, Database::db_escape_string($key), Database::db_escape_string($value));
                            $this->Log->logDebug("Insert Query is " . $query);
                            $result = Database::db_query($this->conn, $this->Log, $query);
                        } else {
                            $result = true;
                        }
                    } else {
                        $this->Log->logDebug("Error saving preference information");
                        break;
                    }
                }
            }
        }
        return $result;
    }

    public function delete($userName, $options = null)
    {
        $result = false;

        if ($userName === null) {
            $result = array(
                'result' => false,
                'message' => 'Username is not optional for deletion.'
            );
        } else {
            if (is_array($options)) {
                foreach ($options as $key => $value) {
                    $query = "DELETE FROM mgportal.preferences WHERE ";
                    $query .= "key=" . Database::db_escape_string($key);
                    $query .= " AND username=" . Database::db_escape_string($userName) . ";";
                    $this->Log->logDebug("Query is " . $query);
                    $result = Database::db_query($this->conn, $this->Log, $query);
                    if ($result !== false) {
                        $result = true;
                    } else {
                        $this->Log->logDebug("Error deleting preference key for user");
                        break;
                    }
                }
            } else {
                $query = "DELETE FROM mgportal.preferences WHERE ";
                $query .= " username='" . $userName . "';";
                $this->Log->logDebug("Query is " . $query);
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logDebug("Error deleting all user preferences.");
                }
            }
        }
        return $result;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $preferences = new Preferences($conn, $Log);
        $which = isset($_GET['user']) ? $_GET['user'] : null;
        $key = isset($_GET['key']) ? $_GET['key'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $preferences->get($which, $key);
                //$result = $users->forgotPassword(1);
                $http->buildResult($result, "get");
                break;
            case 'put':
                $result = $preferences->update($which, $http->getData());
                $http->buildResult($result, "put");
                break;
            case 'delete':
                $result = $preferences->delete($which, $key);
                $http->buildResult($result, 'delete');
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}
?>