<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class ProtectionHistory extends BaseAPI {

    const DAYS_TO_KEEP = 30;
    const ENTRIES_PER_DAY = 1;

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which = null, $start_time = null, $end_time = null, $isReport = false){
        $query = "SELECT p.id, p.device_id, p.date, p.vm_count, p.server_count, p.app_count, p.gb_used, p.vm_replicated, p.all_replicated, ";
        $query .= "d.hostname, d.unique_identifier FROM mgportal.protected p INNER JOIN mgportal.devices d ON p.device_id=d.id";
        $which_conjunction = " WHERE ";
        if ($start_time != null) {
            $query .= " WHERE date >= " . $start_time . " ";
            $which_conjunction = " AND ";
        }
        if ($end_time != null) {
            $query .= $which_conjunction . " date <= " . $end_time . " ";
            $which_conjunction = " AND ";
        }
        if ($which !== null) {
            $query .= $which_conjunction . "device_id=" . $which;
        } else {
            $query .= " ORDER by device_id ASC, date DESC";
        }

        $protected = array();

        $this->Log->logDebug('Query: ' . $query);
        $dbResult = Database::db_query($this->conn, $this->Log, $query);
        if ($dbResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbResult)){
                $timestamp = (int)$row[2];
                $dateString = $dateString = date('m/d/Y', $timestamp);
                $entry = array(
                    'id' => $row[0],
                    'device_id' => $row[1],
                    'date' => (int)$row[2],
                    'date_string' => $dateString,
                    'vm_count' => $row[3],
                    'server_count' => $row[4],
                    'app_count' => $row[5],
                    'gb_used' => (float)$row[6],
                    'vm_replicated' => $row[7],
                    'all_replicated' => $row[8],
                    'name' => $row[9],
                    'uuid' => $row[10]);

                $protected[] = $entry;
            }
        }
        return $isReport ? array("data" => $protected) : array("protection" => $protected);
    }

    /*
     * Returns the timestamp of the end of the specified day.
     */
    public function getEndOfDay($end_date) {
        $endTime = null;
        if ($end_date != null) {
            $endTime = strtotime($end_date . ',11:59:59pm');
            if ($endTime === false) {
                $endTime = strtotime($end_date);
            }
            if ($endTime === false) {
                $endTime = null;
            }
        }
        return $endTime;
    }

    public function purgeProtected($days = ProtectionHistory::DAYS_TO_KEEP, $entriesPerDay = ProtectionHistory::ENTRIES_PER_DAY){
        $protection = $this->get(null);
        $entries = $protection['protection'];

        $dayOfYear = -1;
        $deviceID = -1;
        $countPerDay = 1;

        $idsToDelete = array();
        $nowDayOfYear = date('z', time());
        foreach ($entries as $entry) {
            $entryDayOfYear = date('z', $entry['date']);
            if ($entry['device_id'] != $deviceID) {
                // keep the first (most recent) for each device.
                $deviceID = $entry['device_id'];
                $dayOfYear = $entryDayOfYear;
                continue;
            }
            if ($nowDayOfYear - $entryDayOfYear >= $days) {
                $this->Log->logDebug('Purging id ' . $entry['id'] . ' as it is older than ' . $days . ' days.');
                // delete if > days older.
                $idsToDelete[] = $entry['id'];
            } else {
                if ($entryDayOfYear == $dayOfYear) {
                    $countPerDay++;
                    if ($countPerDay > $entriesPerDay) {
                        $this->Log->logDebug('Purging id ' . $entry['id'] . ' as there are greater than ' . $entriesPerDay . ' for day ' . $dayOfYear);
                        $idsToDelete[] = $entry['id'];
                    }
                } else {
                    $countPerDay = 1;
                    $dayOfYear = $entryDayOfYear;
                }
            }
        }
        if (count($idsToDelete) > 0) {
            // Delete IDs.
            $query = "DELETE FROM mgportal.protected WHERE id IN (" . implode(',', $idsToDelete) . ")";
            $dbResult = Database::db_query($this->conn, $this->Log, $query);
            if ($dbResult !== false) {
                $result = array('result' => true, 'message' => 'Purged ' . count($idsToDelete) . ' data points.');
            } else {
                $result = false;
            }
        } else {
            $result = array('result' => true, 'message' => 'No data protection entries were deleted.');
        }
        return $result;
    }
};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $protected = new ProtectionHistory($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $start_date = isset($_GET['start_date']) ? $_GET['start_date'] : null;
                $start_time = $start_date != null ? strtotime($start_date) : null;
                $end_date = isset($_GET['end_date']) ? $_GET['end_date'] : null;
                $end_time = $protected->getEndOfDay($end_date);

                $isReport = isset($_GET['type']) && $_GET['type'] == 'report';
                $result = $protected->get($which, $start_time, $end_time, $isReport);
                $http->buildResult($result, "get");
                break;
            case 'delete':
                $days = isset($_GET['days']) ? $_GET['days'] : ProtectionHistory::DAYS_TO_KEEP;
                $entriesPerDay = isset($_GET['perday']) ? $_GET['perday'] : ProtectionHistory::ENTRIES_PER_DAY;
                $result = $protected->purgeProtected($days, $entriesPerDay);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}

?>