<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class RabbitMQ extends BaseAPI {
    const DEFAULT_LOCATION = 1;
    const FILENAME = '/usr/local/mgportal/conf.d/mgportal.conf';
    const JSON_PRETTY_PRINT = 128;      // option to json_encode, only if PHP 5.4 or higher
    const CERTS_DIR = '/usr/local/mgportal/certs';

    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    private function lookupDevice($asset) {
        $found = false;
        $query = "SELECT id FROM mgportal.devices WHERE unique_identifier='" . $asset . "'";
        $deviceResult = Database::db_query($this->conn, $this->Log, $query);
        if ($deviceResult !== false && Database::db_num_rows($this->Log, $deviceResult) === 1){
            $found = true;
        }
        return $found;
    }

    public function authenticate($username, $password, $vhost, $resource, $name, $permission){
        $result = "deny";

        if ($username === null) {
            $result = array('result' => false, 'message' => 'username must be provided.');
            return $result;
        }

        if ($username !== '') {
            if ($this->lookupDevice($username)) {
                $result = "allow";
            }
        }

        if ($result == "allow") {
            if ($password != null) {
                if ($password !== Configuration::OBFUSCATED_PW) {
                    $result = "deny";
                }
            } else {
                if ($resource === null && $name === null && $permission === null) {
                    // no matter the vhost, okay.
                    ;
                } else {
                    // look for a resource type and queue name
                    $valid = $resource == 'queue' && strstr($name, 'route.') !== false && strstr($name, $username) !== false;
                    if (!$valid) {
                        $valid = $resource == 'exchange' && (strstr($name, 'amq.default') !== false || (strstr($name, 'amq.direct') !== false) || ($name == 'cloud-default'));
                        if (!$valid) {
                            $result = "deny";
                        }
                    }
                }
            }
        }

        return $result;
    }

    public function getClientKeys() {
        $result = array('result' => true);
        $ca = file_get_contents(RabbitMQ::CERTS_DIR . '/testca/cacert.pem');
        if ($ca !== false) {
            $result['ca'] = $ca;
            $cert = file_get_contents(RabbitMQ::CERTS_DIR . '/client/cert.pem');
            if ($cert !== false) {
                $result['cert'] = $cert;
                $private = file_get_contents(RabbitMQ::CERTS_DIR . '/client/key.pem');
                if ($private !== false) {
                    $result['private'] = $private;
                } else {
                    $result = array('result' => false, 'message' => 'Error getting client private key');
                }
            } else {
                $result = array('result' => false, 'message' => 'Error getting client certificate');
            }
        } else {
            $result = array('result' => false, 'message' => 'Error getting CA file');
        }
        return $result;
    }

    /*
     * Returns server status, either 'down' or 'up'.
     */
    public function getServerStatus() {
        $result = array('result' => true);
        exec('ps -efal | grep rabbitmq-server | grep -v grep', $output, $returnValue);
        if ($returnValue != 0) {
            $result['status'] = 'down';
        } else {
            $result['status'] = 'up';
        }
        return $result;
    }

    /*
     * Checks the server connection for the specified Asset, first checks to see if the server is up.
     */
    public function getConnectionStatus($connection) {
        $result = array('result' => true);
        $serverStatus = $this->getServerStatus();
        if ($serverStatus['status'] == 'up') {
            $result['server'] = true;
            $cmd = 'sudo -S /usr/sbin/rabbitmqctl list_connections | grep "' . $connection . '" | grep -v "127.0.0.1"';
            exec($cmd, $output, $returnValue);
            if ($returnValue == 0) {
                $result['connection'] = true;
            } else {
                $result['connection'] = false;
            }
        } else {
            $result['server'] = false;
            $result['connection'] = false;
        }
        return $result;
    }

    public function updateServerStatus($toState, $rootPassword) {
        if ($this->valid_credentials('root', $rootPassword)) {
            if ($toState == 'down') {
                $toState = 'stop';
            } else if ($toState == 'up') {
                $toState = 'start';
            }
            $command = sprintf("sudo -S /bin/bash /etc/init.d/rabbitmq-server %s", $toState);
            $handle = popen($command, 'w');
            fwrite($handle, "$rootPassword\n");
            pclose($handle);
            $result = array('result' => true);
        } else {
            $result = array('result' => false, 'message' => 'Invalid password for root user.');
        }
        return $result;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $rabbit = new RabbitMQ($conn, $Log);
        $method = $http->getMethod();
        switch ($http->getMethod()) {
            case 'post':
            case 'put':
                $data = $http->getData();
                break;
            case 'get':
                $data = $_GET;
                break;
        }
        $vhost = isset($data['vhost']) ? $data['vhost'] : null;
        $resource = isset($data['resource']) ? $data['resource'] : null;
        $name = isset($data['name']) ? $data['name'] : null;
        $permission = isset($data['permission']) ? $data['permission'] : null;
        $username = isset($data['username']) ? $data['username'] : null;
        $password = isset($data['password']) ? $data['password'] : null;
        switch ($method) {
            case 'get':
                if (isset($data['client_keys'])) {
                    $result = $rabbit->getClientKeys();
                    $http->buildResult($result, "get");
                } else if (isset($data['type'])) {
                    if ($data['type'] == 'status') {
                        $result = $rabbit->getServerStatus();
                        $http->buildResult($result, "get");
                    } else if ($data['type'] == 'connection' && isset($data['asset'])) {
                        $asset = $data['asset'];
                        $result = $rabbit->getConnectionStatus($asset);
                        $http->buildResult($result, "get");
                    } else {
                        $http->buildFailure(false, "bad rabbitmq command, must specify server status or connection status for an asset");
                    }
                } else {
                    $result = $rabbit->authenticate($username, $password, $vhost, $resource, $name, $permission);
                    if ($http->isSuccessful($result)) {
                        $http->buildTextResult($result, "put");
                    } else {
                        $http->buildResult($result, "put");
                    }
                }
                break;
            case 'put':
                if (isset($data['action'])) {
                    if (isset($data['password'])) {
                        $toState = $data['action'];
                        $rootPassword = $data['password'];
                        $result = $rabbit->updateServerStatus($toState, $rootPassword);
                        $http->buildResult($result, "put");
                    } else {
                        $result = array('result' => 'false', 'message' => 'You must provide the root password to start or stop the messaging service.');
                        $http->buildFailure(false, "You must provide the root password to start or stop the messaging service.");
                    }
                }
                break;
            case 'post':
                $result = $rabbit->authenticate($username, $password, $vhost, $resource, $name, $permission);
                if ($http->isSuccessful($result)) {
                    $http->buildTextResult($result, "put");
                } else {
                    $http->buildResult($result, "put");
                }
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}


?>
