<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Reports extends BaseAPI{
    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($type, $format, $limit = 1){
        $result = "";
        switch ($type){
            case 'Monthly':
                $result = $this->get_monthly($format, $limit);
                break;
            case 'Protected':
                $result = $this->get_protected($format, $limit);
                break;
            case 'Managers':
                $result = $this->get_customerDevices($format);
                break;
        }
        return $result;
    }

    private function get_monthly($format, $limit) {
        $result = $format == 'csv' ? "" : array();
        $assets = Database::db_query($this->conn, $this->Log, "SELECT id, unique_identifier, license_features, install_date from mgportal.devices");
        if ($assets !== false) {
            if ($format == 'csv') {
                $result .= "Latest Data Date,Asset Tag,Feature String,Install Date, Protected Assets (by Query), Protected Assets (by API)\n";              
            }
            while ($row = Database::db_fetch_row($this->Log,  $assets)) {
                //printf("asset = %s\n", $row[1]);
                $device_id = $row[0];
                $asset = $row[1];
                $feature_string = $row[2];
                $install_date = $row[3];

                $query = "SELECT date, protected_assets_by_query, protected_assets_by_api FROM mgportal.monthly where device_id=" . $device_id . " ORDER BY date DESC";
                if ($limit != "none" || (int)$limit > 0) {
                    $query .= " LIMIT " . $limit;
                }
                $protected = Database::db_query($this->conn, $this->Log, $query);
                if ($protected !== false) {
                    while ($item = Database::db_fetch_row($this->Log,  $protected)) {
                        $timestamp = $item[0];
                        $dateString = date('m/d/Y', $timestamp);
                        if ($format == 'csv') {
                            $line = sprintf("\"%s\",\"%s\",\"%s\",\"%s\",%d,%d\n",
                                $dateString, $asset, $feature_string, $install_date,
                                $item[1], $item[2]);
                            $result .= $line;                           
                        } else {
                            $line = array('date' => $dateString,
                                            'asset' => $asset,
                                            'feature' => $feature_string,
                                            'installed' => $install_date,
                                            'assets_by_query' => $item[1],
                                            'assets_by_api' => $item[2]
                                    );
                            $result[] = $line;
                        }
                    }
                }
            }
        }
        return $result;
    }

    private function get_protected($format, $limit) {
        $result = $format == 'csv' ? "" : array();
        $assets = Database::db_query($this->conn, $this->Log, "SELECT id, unique_identifier, license_features, install_date from mgportal.devices");
        if ($assets !== false) {
            //printf("'Latest Data Date','Asset Tag','Feature String','Install Date', 'VM Count', 'Server Count', 'App Count', 'GB Used', 'VM Replicating', 'All Replicating'\n");
            if ($format == 'csv') {
                $result .= "Latest Data Date,Asset Tag,Feature String,Install Date, VM Count, Server Count, App Count, GB Used, VM Replicating, All Replicating\n";
            }
            while ($row = Database::db_fetch_row($this->Log,  $assets)) {
                //printf("asset = %s\n", $row[1]);
                $device_id = $row[0];
                $asset = $row[1];
                $feature_string = $row[2];
                $install_date = $row[3];

                $query = "SELECT date, vm_count, server_count, app_count, gb_used, vm_replicated, all_replicated FROM mgportal.protected where device_id=" . $device_id . " ORDER BY date DESC";
                if ($limit != "none" || (int)$limit > 0) {
                    $query .= " LIMIT " . $limit;
                }
                $protected = Database::db_query($this->conn, $this->Log, $query);
                if ($protected !== false) {
                    while ($item = Database::db_fetch_row($this->Log,  $protected)) {
                        $timestamp = $item[0];
                        $dateString = date('m/d/Y', $timestamp);
                        if ($format == 'csv') {
                            $line = sprintf("\"%s\",\"%s\",\"%s\",\"%s\",%d,%d,%d,%.2f,%d,%d\n",
                                $dateString, $asset, $feature_string, $install_date,
                                $item[1], $item[2], $item[3], $item[4],
                                $item[5], $item[6]);
                            $result .= $line;
                        } else {
                            $line = array('date' => $dateString,
                                'asset' => $asset,
                                'feature' => $feature_string,
                                'installed' => $install_date,
                                'vm_count' => $item[1],
                                'server_count' => $item[2],
                                'app_count' => $item[3],
                                'gb_used' => $item[4],
                                'vm_replicated' => $item[5],
                                'all_replicated' => $item[6]
                            );
                            $result[] = $line;
                        }
                    }
                }
            }
        }
        return $result;
    }

    public function get_customerDevices($format) {
        $result = array();
        // get managers with locations.
        $query = "SELECT m.id, m.name, l.id, l.name, d.id, d.unique_identifier, d.hostname FROM mgportal.managers m INNER JOIN mgportal.locations l ";
        $query .= "ON m.id=l.manager_id INNER JOIN mgportal.devices d ON l.id=d.location_id";
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $device_id = $row[4];
                $item = array('manager_id' => $row[0],
                    'manager' => $row[1],
                    'location_id' => $row[2],
                    'location' => $row[3],
                    'device_id' => $device_id,
                    'uuid' => $row[5],
                    'hostname' => $row[6]);

                $query = "SELECT date, vm_count, server_count, app_count, gb_used, vm_replicated, all_replicated FROM mgportal.protected ";
                $query .= "WHERE device_id=" . $device_id . " ORDER BY date DESC LIMIT 1";
                $protected = Database::db_query($this->conn, $this->Log, $query);
                if ($protected !== false) {
                    if ($row = Database::db_fetch_row($this->Log,  $protected)) {
                        $timestamp = $row[0];
                        $dateString = date('m/d/Y', $timestamp);
                        $line = array('date' => $dateString,
                            'vm_count' => $row[1],
                            'server_count' => $row[2],
                            'app_count' => $row[3],
                            'gb_used' => $row[4],
                            'vm_replicated' => $row[5],
                            'all_replicated' => $row[6]
                        );
                        $item = array_merge($item, $line);
                    }
                }

                $result[] = $item;
            }
        }
        return $result;
    }
};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $reports = new Reports($conn, $Log);
        $which = isset($_GET['type']) ? $_GET['type'] : null;
        $limit = isset($_GET['limit']) ? $_GET['limit'] : 1;
        $format = isset($_GET['format']) ? $_GET['format'] : 'json';
        switch ($http->getMethod()) {
            case 'get':
                $result = $reports->get($which, $format, $limit);
                if (is_string($result)) {
                    $jsonResult = array('output' => $result, 'display' => 'csv');
                } else {
                    $jsonResult = array('output' => $result, 'display' => 'json');
                }
                $http->buildResult($jsonResult, "get");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}
