<?php

// helper classes do not need constants / defines

class Sessions {
    const EXPIRED_TOKEN = "Expired";
    
    public function __construct($conn = NULL, $Log = NULL) {
        $this->conn = $conn;
        $this->Log = $Log;
    }
    
    public function newSession($userId, $time){
        $token = false;

        $expires = $time + (60 * 60 * 7 * 24); // 1 week out
        $uuid = $this->createGUID();

        $str = "INSERT INTO mgportal.sessions (userid, uuid, expiry)";
        $str .= " VALUES (%d, '%s', %d);";
        $query = sprintf($str, $userId, $uuid, $expires);

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            //$row = Database::db_fetch_row($this->Log,  $result);
            //$session_id = $row[0];
            $cookie = $userId . ":" . $uuid . ":" . $expires;
            $token = base64_encode($cookie);
        } else {
            $this->Log->logError("Error saving session");
        }
        return $token;
    }

    private function createGUID(){
        if (function_exists('com_create_guid') === true){
            return trim(com_create_guid(), '{}');
        }
        return sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X',
               mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535),
               mt_rand(16384, 20479), mt_rand(32768, 49151),
               mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));

    }
    
    public function doesUserSessionExist($userid){
        if ($this->conn !== NULL) {
            $query = "SELECT userid, uuid, expiry FROM mgportal.sessions";
            $query .= " WHERE userid=" . $userid . ";";
            $dbSessions = Database::db_query($this->conn, $this->Log, $query);
            if ($dbSessions !== false){
                return true;
            }
        } else {
            $message = "no database connection open to check user session";
            if ($this->Log !== NULL) {
                $this->Log->logError($message);
            } else {
                global $Log;
                $Log->logError($message);
            }
        }
        return false;
    }

    public function isTokenValid($inToken){
        
        $valid = false;     // initialize as an invalid token.
        
        // deconstruct token first
        $token = base64_decode($inToken);
        $token = explode(":",$token);
        $inUser = $token[0];
        $inGUID = $token[1];
        $inExpiry = $token[2];

        $query = "SELECT userid, uuid, expiry FROM mgportal.sessions";
        $query .= " WHERE userid=" . $inUser . " AND uuid='" . $inGUID;
        $query .= "' AND expiry=" . $inExpiry . ";";


        $db = new Database();
        $conn = $db->connect();
        if ($conn !== false) {
            $dbSessions = Database::db_query($conn, $this->Log, $query);
            if ($dbSessions !== false){
                $row = Database::db_fetch_row($this->Log,  $dbSessions);
                if ($row !== ""){
                    $expires = $row[2];
                    if (time() < $expires){
                        $valid = true;
                    } else {
                        $valid = Sessions::EXPIRED_TOKEN;
                    }
                }
            }
            $db->closeConnection();
        } else {
            $valid = Database::CONNECT_ERROR;
        }
        return $valid;
    }

    public function deleteSession($inToken){
        $token = base64_decode($inToken);
        $token = explode(":",$token);
        $inUser = $token[0];
        $inGUID = $token[1];
        $inExpiry = $token[2];

        $query = "DELETE FROM mgportal.sessions WHERE userid=";
        $query .= $inUser;
        $query .= " AND uuid='";
        $query .= $inGUID . "';";

        $result = false;
        $db = new Database();
        $conn = $db->connect();
        if ($conn !== false) {
            $result = Database::db_query($conn, $this->Log, $query);
            if ($result !== false){
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row === null){
                    $result = array(
                        'result' => false,
                        'message' => "Error: could not find session in session table"
                    );
                } else {
                    $result = true;
                }
            } else {
                $message = "Error deleting session for " . $inUser;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
            $db->closeConnection();
        }
        return $result;
    }

    public function deleteUserSessions($inUser){
        $query = "DELETE FROM mgportal.sessions WHERE userid=";
        $query .= $inUser;
        $query .= ";";

        $result = false;
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $result);
            if ($row === null){
                $result = array(
                    'result' => false,
                    'message' => "Error: could not find session for the provided userid"
                );
            } else {
                $result = true;
            }
        } else {
            $message = "Error deleting session for " . $inUser;
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
        }
        return $result;
    }

    public function deleteUserSession($token) {

        $token = base64_decode($token);
        $token = explode(":",$token);
        $inUser = $token[0];
        $inGUID = $token[1];

        $query = "DELETE FROM mgportal.sessions WHERE userid=";
        $query .= $inUser;
        $query .= " AND uuid='";
        $query .= $inGUID . "';";

        $this->Log->logInfo("Delete session");
        $this->Log->logInfo($query);

        $result = false;
        if ($this->conn !== NULL) {
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false){
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row === null){
                    $result = array(
                        'result' => false,
                        'message' => "Error: could not find session in session table"
                    );
                } else {
                    $result = true;
                }
            } else {
                $message = "Error deleting session for " . $inUser;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        }
        return $result;
    }

    public function getUserForToken($inToken) {

        // deconstruct token first
        $token = base64_decode($inToken);
        $token = explode(":",$token);
        $inUser = $token[0];
        $inGUID = $token[1];
        $inExpiry = $token[2];

        return $inUser;
    }

    public function getUserSessionForToken($inToken) {
        $result = false;
        $userid = $this->getUserForToken($inToken);
        if ($userid > 0) {
            $query = "SELECT privileges, company_id FROM mgportal.users WHERE user_id=";
            $query .= $userid . ";";

            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false){
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row !== null) {
                    $result = array();
                    $result['privilege'] = (int)$row[0];
                    $result['company_id'] = (int)$row[1];
                    $result['user_id'] = $userid;
                }
            } else {
                $message = "Error finding privilege for " . $userid;
                $this->Log->logError($message);
            }
        }
        return $result;
    }
};

?>
